#!/usr/bin/env python

# CAVEAT UTILITOR
#
# This file was automatically generated by TatSu.
#
#    https://pypi.python.org/pypi/tatsu/
#
# Any changes you make to it will be overwritten the next time
# the file is generated.

from __future__ import annotations

from typing import Any
from dataclasses import dataclass

from tatsu.objectmodel import Node
from tatsu.semantics import ModelBuilderSemantics


@dataclass(eq=False)
class ModelBase(Node):
    pass


class TextGrammarModelBuilderSemantics(ModelBuilderSemantics):
    def __init__(self, context=None, types=None):
        types = [
            t for t in globals().values()
            if type(t) is type and issubclass(t, ModelBase)
        ] + (types or [])
        super().__init__(context=context, types=types)


@dataclass(eq=False)
class Literal(ModelBase):
    value: Any = None


@dataclass(eq=False)
class AdjectiveNoun(ModelBase):
    adjective: Any = None
    noun: Any = None


@dataclass(eq=False)
class Match(ModelBase):
    lhs: Any = None
    rhs: Any = None


@dataclass(eq=False)
class ProductionRule(ModelBase):
    alternatives: Any = None
    symbol: Any = None


@dataclass(eq=False)
class TextGrammar(ModelBase):
    rules: Any = None
